#include "M3AP-PDU-Descriptions.h"
#include "M3AP-PDU-Contents.h"
#include "M3AP-Constants.h"
#include "rtxsrc/rtxCommon.h"
#include "rtxsrc/rtxFile.h"

/* methods to populate types with test data */
InitiatingMessage  *pop_InitMsg (OSCTXT *pctxt);
ErrorIndication_protocolIEs_element *pop_CapElem (OSCTXT *pctxt);

int main (int argc, char** argv)
{
   M3AP_PDU   pdu;
   OSCTXT     ctxt;
   OSOCTET*   msgptr;
   int        i, len, stat;
   OSBOOL     aligned = TRUE;
   OSBOOL     trace = TRUE;
   OSBOOL     verbose = FALSE;
   const char* filename = "message.dat";

   /* Process command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 1;
         }
      }
   }

   /* Initialize context structure */
   stat = rtInitContext (&ctxt);
   if (stat != 0) {
      rtxErrPrint (&ctxt);
      return stat;
   }
   rtxSetDiag (&ctxt, verbose);
   pu_setBuffer (&ctxt, 0, 0, aligned);
   pu_setTrace (&ctxt, trace);

   pdu.t = T_M3AP_PDU_initiatingMessage;
   pdu.u.initiatingMessage = pop_InitMsg (&ctxt);

   if (trace) {
      printf ("The following record will be encoded:\n");
      asn1Print_M3AP_PDU ("M3AP_PDU", &pdu);
      printf ("\n");
   }

   /* Encode */
   if ((stat = asn1PE_M3AP_PDU (&ctxt, &pdu))== 0) {
      if (trace) {
         printf ("Encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         pu_hexdump (&ctxt);
         printf ("Binary dump:\n");
         pu_bindump (&ctxt, "M3AP_PDU");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      rtFreeContext (&ctxt);
      return -1;
   }
   msgptr = pe_GetMsgPtr (&ctxt, &len);

   /* Write the encoded message out to the output file */
   stat = rtxFileWriteBinary (filename, msgptr, len);
   if (stat < 0) {
      printf ("Error opening %s for write access\n", filename);
   }

   rtFreeContext (&ctxt);

   return stat;
}

InitiatingMessage *pop_InitMsg (OSCTXT *pctxt) 
{
   OSRTDList* plist;
   ErrorIndication_protocolIEs_element* pdata;
   InitiatingMessage* pvalue = rtxMemAllocType (pctxt, InitiatingMessage);
   asn1Init_InitiatingMessage (pvalue);

   pvalue->procedureCode = ASN1V_id_errorIndication;
   pvalue->criticality = Criticality_reject;

   pvalue->value.t = T_M3PDUDescr_SEP_errorIndication;
   pvalue->value.u.errorIndication = 
      rtxMemAllocType (pctxt, ErrorIndication);

   asn1Init_ErrorIndication 
      (pvalue->value.u.errorIndication);

   plist = &pvalue->value.u.errorIndication->protocolIEs;

   rtxDListInit (plist);

   pdata = pop_CapElem (pctxt);

   rtxDListAppend (pctxt, plist, (void*)pdata);

   return (pvalue);
}

ErrorIndication_protocolIEs_element *pop_CapElem (OSCTXT *pctxt)
{
   ErrorIndication_protocolIEs_element* pvalue = 
      rtxMemAllocType (pctxt, ErrorIndication_protocolIEs_element);

   asn1Init_ErrorIndication_protocolIEs_element (pvalue);

   pvalue->id = ASN1V_id_MME_MBMS_M3AP_ID;
   pvalue->criticality = Criticality_reject;
   pvalue->value.t = 
      T_M3PDUCont_ErrorIndication_IEs_id_MME_MBMS_M3AP_ID;
   pvalue->value.u._ErrorIndication_IEs_id_MME_MBMS_M3AP_ID = 16320995;
   
   return (pvalue);
}

